@@ -6,7 +6,7 @@
 #include <fstream>
 #include <stdexcept>
 
-#include "../compatability_macros.h"
+#include "../compatability/tr1_usage.h"
 
 #if USE_TR1 > 0
 #include <tr1/type_traits>
