@@ -128,6 +128,60 @@ public:
 		string_stream_ << object_typer_.label << " " << remove_unsafe_characters_in_string(label) << " ";
 	}
 
+	void add_as_wireframe(const std::string& str)
+	{
+		std::istringstream input(str);
+		std::ostringstream output;
+		while(input.good())
+		{
+			std::string type_str;
+			input >> type_str;
+			const ObjectTypeMarker type(type_str, object_typer_);
+			if(type.tstrip || type.tfan || type.tfanc)
+			{
+				std::vector<PlainPoint> vertices;
+				std::vector<PlainPoint> normals;
+				if(read_strip_or_fan_from_stream(type.tstrip, type.tfan, type.tfanc, input, vertices, normals) && vertices.size()>=3)
+				{
+					if(type.tstrip)
+					{
+						if(vertices.size()==3)
+						{
+							output << object_typer_.lloop << " ";
+							write_points_vector_to_stream(vertices, output);
+						}
+						else
+						{
+							output << object_typer_.lstrip << " 2 ";
+							write_point_to_stream(vertices[0], output);
+							write_point_to_stream(vertices[1], output);
+							for(std::size_t i=0;(i+2)<vertices.size();i++)
+							{
+								output << object_typer_.lstrip << " 3 ";
+								write_point_to_stream(vertices[i], output);
+								write_point_to_stream(vertices[i+2], output);
+								write_point_to_stream(vertices[i+1], output);
+							}
+						}
+					}
+					else
+					{
+						output << object_typer_.lloop << " " << (vertices.size()-1) << " ";
+						for(std::size_t i=1;i<vertices.size();i++)
+						{
+							write_point_to_stream(vertices[i], output);
+						}
+					}
+				}
+			}
+			else
+			{
+				output << type_str << " ";
+			}
+		}
+		add(output.str());
+	}
+
 	std::string str() const
 	{
 		return string_stream_.str();
@@ -334,6 +388,130 @@ public:
 		}
 	}
 
+	bool write_to_low_level_triangle_buffers(std::vector<float>& output_vertices, std::vector<float>& output_normals, std::vector<unsigned int>& output_indices, const bool indexed) const
+	{
+		output_vertices.clear();
+		output_normals.clear();
+		output_indices.clear();
+
+		std::istringstream input(str());
+		if(!input.good())
+		{
+			return false;
+		}
+
+		std::vector<PlainPoint> global_vertices;
+		std::vector<PlainPoint> global_normals;
+		std::vector<PlainTriple> global_triples;
+		std::vector< std::pair<PlainPoint, double> > global_spheres;
+
+		{
+			Color color(0xFFFFFF);
+			std::string label="undefined";
+			while(input.good())
+			{
+				std::string type_str;
+				input >> type_str;
+				const ObjectTypeMarker type(type_str, object_typer_);
+				if(type.color || type.label)
+				{
+					if(type.color)
+					{
+						color=read_color_from_stream(input);
+					}
+					else if(type.label)
+					{
+						input >> label;
+					}
+				}
+				else if(type.tstrip || type.tfan || type.tfanc)
+				{
+					std::vector<PlainPoint> vertices;
+					std::vector<PlainPoint> normals;
+					if(read_strip_or_fan_from_stream(type.tstrip, type.tfan, type.tfanc, input, vertices, normals) && vertices.size()==normals.size())
+					{
+						const std::size_t offset=global_vertices.size();
+						global_vertices.insert(global_vertices.end(), vertices.begin(), vertices.end());
+						global_normals.insert(global_normals.end(), normals.begin(), normals.end());
+						for(std::size_t i=0;(i+2)<vertices.size();i++)
+						{
+							global_triples.push_back(PlainTriple(offset+(type.tstrip ? i : 0), offset+(i+1), offset+(i+2)));
+						}
+					}
+				}
+				else if(type.sphere)
+				{
+					const PlainPoint c=read_point_from_stream(input);
+					double r;
+					input >> r;
+					global_spheres.push_back(std::make_pair(c, r));
+				}
+			}
+		}
+
+		if(global_triples.empty() || global_vertices.empty() || global_vertices.size()!=global_normals.size())
+		{
+			return false;
+		}
+
+		if(indexed)
+		{
+			output_vertices.reserve(global_vertices.size()*3);
+			output_normals.reserve(global_normals.size()*3);
+	        output_indices.reserve(global_triples.size()*3);
+
+	        for(std::size_t i=0;i<global_vertices.size();i++)
+	        {
+	            output_vertices.push_back(global_vertices[i].x);
+	            output_vertices.push_back(global_vertices[i].y);
+	            output_vertices.push_back(global_vertices[i].z);
+	            output_normals.push_back(global_normals[i].x);
+	            output_normals.push_back(global_normals[i].y);
+	            output_normals.push_back(global_normals[i].z);
+	        }
+
+	        for(std::size_t i=0;i<global_triples.size();i++)
+	        {
+	        	output_indices.push_back(global_triples[i].a);
+	        	output_indices.push_back(global_triples[i].b);
+	        	output_indices.push_back(global_triples[i].c);
+	        }
+		}
+		else
+		{
+			output_vertices.reserve(global_triples.size()*9);
+			output_normals.reserve(global_triples.size()*9);
+
+			for(std::size_t i=0;i<global_triples.size();i++)
+			{
+				const PlainTriple& t=global_triples[i];
+				if(t.a<global_vertices.size() && t.b<global_vertices.size() && t.c<global_vertices.size())
+				{
+					output_vertices.push_back(static_cast<float>(global_vertices[t.a].x));
+					output_vertices.push_back(static_cast<float>(global_vertices[t.a].y));
+					output_vertices.push_back(static_cast<float>(global_vertices[t.a].z));
+					output_vertices.push_back(static_cast<float>(global_vertices[t.b].x));
+					output_vertices.push_back(static_cast<float>(global_vertices[t.b].y));
+					output_vertices.push_back(static_cast<float>(global_vertices[t.b].z));
+					output_vertices.push_back(static_cast<float>(global_vertices[t.c].x));
+					output_vertices.push_back(static_cast<float>(global_vertices[t.c].y));
+					output_vertices.push_back(static_cast<float>(global_vertices[t.c].z));
+					output_normals.push_back(static_cast<float>(global_normals[t.a].x));
+					output_normals.push_back(static_cast<float>(global_normals[t.a].y));
+					output_normals.push_back(static_cast<float>(global_normals[t.a].z));
+					output_normals.push_back(static_cast<float>(global_normals[t.b].x));
+					output_normals.push_back(static_cast<float>(global_normals[t.b].y));
+					output_normals.push_back(static_cast<float>(global_normals[t.b].z));
+					output_normals.push_back(static_cast<float>(global_normals[t.c].x));
+					output_normals.push_back(static_cast<float>(global_normals[t.c].y));
+					output_normals.push_back(static_cast<float>(global_normals[t.c].z));
+				}
+			}
+		}
+
+		return (!output_vertices.empty());
+	}
+
 private:
 	struct ObjectTyper
 	{
@@ -457,7 +635,7 @@ private:
 				color.r=static_cast<unsigned char>(255*(value/0.5));
 				color.g=color.r;
 			}
-			else if(value>0.5)
+			else
 			{
 				color.r=255;
 				color.b=static_cast<unsigned char>(255*(1-(value-0.5)/0.5));
@@ -654,9 +832,9 @@ private:
 			const std::string& id,
 			std::ostream& output)
 	{
-		static int use_num=0;
 		if(!(vertices.empty() || triples.empty()))
 		{
+			static int use_num=0;
 			output << "draw " << id << use_num << " ";
 			output << "POLYGON " << vertices.size() << " ";
 			for(std::size_t i=0;i<vertices.size();i++)
